/* eslint-disable no-magic-numbers */
import { useAppSelector } from '@/redux/hooks/useAppSelector';
import {
  highestZIndexSelector,
  layerByIdSelector,
  lowestZIndexSelector,
} from '@/redux/layers/layers.selectors';
import { JSX, useState } from 'react';
import { LayersDrawerImageItem } from '@/components/Map/Drawer/LayersDrawer/LayersDrawerImageItem.component';
import { LayersDrawerTextItem } from '@/components/Map/Drawer/LayersDrawer/LayerDrawerTextItem.component';
import QuestionModal from '@/components/FunctionalArea/Modal/QuestionModal/QustionModal.component';
import { removeLayerImage, updateLayerImageObject } from '@/redux/layers/layers.thunks';
import { layerDeleteImage, layerUpdateImage } from '@/redux/layers/layers.slice';
import removeElementFromLayer from '@/components/Map/MapViewer/utils/shapes/elements/removeElementFromLayer';
import { showToast } from '@/utils/showToast';
import { SerializedError } from '@reduxjs/toolkit';
import { LayerImage } from '@/types/models';
import { useAppDispatch } from '@/redux/hooks/useAppDispatch';
import { useMapInstance } from '@/utils/context/mapInstanceContext';
import { mapModelIdSelector } from '@/redux/map/map.selectors';
import { mapEditToolsSetLayerObject } from '@/redux/mapEditTools/mapEditTools.slice';
import updateGlyph from '@/components/Map/MapViewer/utils/shapes/elements/Glyph/updateGlyph';
import { useSetBounds } from '@/utils/map/useSetBounds';
import { mapEditToolsLayerImageObjectSelector } from '@/redux/mapEditTools/mapEditTools.selectors';
import { usePointToProjection } from '@/utils/map/usePointToProjection';
import { Coordinate } from 'ol/coordinate';
import { openLayerImageObjectEditFactoryModal } from '@/redux/modal/modal.slice';

interface LayersDrawerObjectsListProps {
  layerId: number;
  isLayerVisible: boolean;
  isLayerActive: boolean;
}

export const LayersDrawerObjectsList = ({
  layerId,
  isLayerVisible,
  isLayerActive,
}: LayersDrawerObjectsListProps): JSX.Element | null => {
  const currentModelId = useAppSelector(mapModelIdSelector);
  const highestZIndex = useAppSelector(highestZIndexSelector);
  const lowestZIndex = useAppSelector(lowestZIndexSelector);
  const layer = useAppSelector(state => layerByIdSelector(state, layerId));
  const mapEditToolsLayerImageObject = useAppSelector(mapEditToolsLayerImageObjectSelector);
  const [isImageRemoveModalOpen, setIsImageRemoveModalOpen] = useState(false);
  const [layerImageToRemove, setLayerImageToRemove] = useState<LayerImage | null>(null);
  const dispatch = useAppDispatch();
  const setBounds = useSetBounds();
  const pointToProjection = usePointToProjection();
  const { mapInstance } = useMapInstance();

  const removeImage = (layerImage: LayerImage): void => {
    setLayerImageToRemove(layerImage);
    setIsImageRemoveModalOpen(true);
  };

  const rejectImageRemove = (): void => {
    setIsImageRemoveModalOpen(false);
  };

  const confirmImageRemove = async (): Promise<void> => {
    if (!layerImageToRemove) {
      return;
    }
    try {
      await dispatch(
        removeLayerImage({
          modelId: currentModelId,
          layerId: layerImageToRemove.layer,
          imageId: layerImageToRemove.id,
        }),
      ).unwrap();
      dispatch(
        layerDeleteImage({
          modelId: currentModelId,
          layerId: layerImageToRemove.layer,
          imageId: layerImageToRemove.id,
        }),
      );
      removeElementFromLayer({
        mapInstance,
        layerId: layerImageToRemove.layer,
        featureId: layerImageToRemove.id,
      });
      showToast({
        type: 'success',
        message: 'The layer image has been successfully removed',
      });
      setIsImageRemoveModalOpen(false);
    } catch (error) {
      const typedError = error as SerializedError;
      showToast({
        type: 'error',
        message: typedError.message || 'An error occurred while removing the layer image',
      });
    }
  };

  const updateImageZIndex = async ({
    zIndex,
    layerImage,
  }: {
    zIndex: number;
    layerImage: LayerImage;
  }): Promise<void> => {
    const newLayerImage = await dispatch(
      updateLayerImageObject({
        modelId: currentModelId,
        layerId: layerImage.layer,
        ...layerImage,
        z: zIndex,
      }),
    ).unwrap();
    if (newLayerImage) {
      dispatch(
        layerUpdateImage({
          modelId: currentModelId,
          layerId: newLayerImage.layer,
          layerImage: newLayerImage,
        }),
      );
      dispatch(mapEditToolsSetLayerObject(newLayerImage));
      updateGlyph(mapInstance, newLayerImage.layer, newLayerImage);
    }
  };

  const bringImageToFront = async (layerImage: LayerImage): Promise<void> => {
    await updateImageZIndex({ zIndex: highestZIndex + 1, layerImage });
  };

  const bringImageToBack = async (layerImage: LayerImage): Promise<void> => {
    await updateImageZIndex({ zIndex: lowestZIndex - 1, layerImage });
  };

  const centerObject = (layerImage: LayerImage): void => {
    if (mapEditToolsLayerImageObject && mapEditToolsLayerImageObject.id === layerImage.id) {
      const point1 = pointToProjection({ x: layerImage.x, y: layerImage.y });
      const point2 = pointToProjection({
        x: layerImage.x + layerImage.width,
        y: layerImage.y + layerImage.height,
      });
      setBounds([point1, point2] as Coordinate[]);
    }
  };

  const editImage = (): void => {
    dispatch(openLayerImageObjectEditFactoryModal());
  };

  if (!layer) {
    return null;
  }

  return (
    <div className={`${isLayerVisible ? 'opacity-100' : 'opacity-40'} flex flex-col gap-1 ps-3`}>
      <QuestionModal
        isOpen={isImageRemoveModalOpen}
        onClose={rejectImageRemove}
        onConfirm={confirmImageRemove}
        question="Are you sure you want to remove the image?"
      />
      {Object.values(layer.texts).map(layerText => (
        <LayersDrawerTextItem layerText={layerText} key={layerText.id} />
      ))}
      {Object.values(layer.images).map(layerImage => (
        <LayersDrawerImageItem
          layerImage={layerImage}
          key={layerImage.id}
          bringToFront={() => bringImageToFront(layerImage)}
          bringToBack={() => bringImageToBack(layerImage)}
          removeObject={() => removeImage(layerImage)}
          centerObject={() => centerObject(layerImage)}
          editObject={() => editImage()}
          isLayerVisible={isLayerVisible}
          isLayerActive={isLayerActive}
        />
      ))}
    </div>
  );
};
