/* eslint-disable no-magic-numbers */
import { UsePointToProjectionResult } from '@/utils/map/usePointToProjection';
import Style from 'ol/style/Style';
import { Point } from 'ol/geom';
import { Feature } from 'ol';
import { FeatureLike } from 'ol/Feature';
import { MapInstance } from '@/types/map';
import { Color } from '@/types/models';
import { TEXT_CUTOFF_SCALE } from '@/components/Map/MapViewer/MapViewer.constants';
import { HorizontalAlign, VerticalAlign } from '@/components/Map/MapViewer/MapViewer.types';
import getTextCoords from '@/components/Map/MapViewer/utils/shapes/text/getTextCoords';
import getTextStyle from '@/components/Map/MapViewer/utils/shapes/text/getTextStyle';
import { rgbToHex } from '@/components/Map/MapViewer/utils/shapes/style/rgbToHex';
import Polygon from 'ol/geom/Polygon';
import getStroke from '@/components/Map/MapViewer/utils/shapes/style/getStroke';
import getStyle from '@/components/Map/MapViewer/utils/shapes/style/getStyle';
import getScaledElementStyle from '@/components/Map/MapViewer/utils/shapes/style/getScaledElementStyle';
import { Stroke } from 'ol/style';

export interface TextProps {
  x: number;
  y: number;
  width: number;
  height: number;
  layer: number;
  zIndex: number;
  text: string;
  fontSize: number;
  fontColor: Color;
  borderColor: Color;
  verticalAlign: VerticalAlign;
  horizontalAlign: HorizontalAlign;
  pointToProjection: UsePointToProjectionResult;
  mapInstance: MapInstance;
}

export default class Text {
  text: string;

  fontSize: number;

  style: Style;

  polygonStyle: Style;

  strokeStyle: Stroke;

  point: Point;

  feature: Feature<Point>;

  constructor({
    x,
    y,
    width,
    height,
    layer,
    zIndex,
    text,
    fontSize,
    fontColor,
    borderColor,
    verticalAlign,
    horizontalAlign,
    pointToProjection,
    mapInstance,
  }: TextProps) {
    this.text = text;
    this.fontSize = fontSize;

    const textCoords = getTextCoords({
      x,
      y,
      height,
      width,
      fontSize,
      verticalAlign,
      horizontalAlign,
      pointToProjection,
    });
    const borderPolygon = new Polygon([
      [
        pointToProjection({ x, y }),
        pointToProjection({ x: x + width, y }),
        pointToProjection({ x: x + width, y: y + height }),
        pointToProjection({ x, y: y + height }),
        pointToProjection({ x, y }),
      ],
    ]);
    this.strokeStyle = getStroke({
      color: rgbToHex(borderColor),
      width: 1,
    });
    this.polygonStyle = getStyle({
      geometry: borderPolygon,
      borderColor,
      fillColor: { rgb: 0, alpha: 0 },
      lineWidth: 1,
      zIndex,
    });

    const textStyle = getTextStyle({
      text,
      fontSize,
      color: rgbToHex(fontColor),
      zIndex,
      horizontalAlign,
    });
    this.point = new Point(textCoords);
    this.style = textStyle;
    this.style.setGeometry(this.point);

    this.feature = new Feature({
      geometry: this.point,
      getScale: (resolution: number): number => {
        const maxZoom = mapInstance?.getView().get('originalMaxZoom');
        if (maxZoom) {
          const minResolution = mapInstance?.getView().getResolutionForZoom(maxZoom);
          if (minResolution) {
            return Math.round((minResolution / resolution) * 100) / 100;
          }
        }
        return 1;
      },
      layer,
    });

    this.feature.setStyle(this.getStyle.bind(this));
  }

  protected getStyle(feature: FeatureLike, resolution: number): Style | Array<Style> | void {
    const getScale = feature.get('getScale');
    let scale = 1;
    if (getScale instanceof Function) {
      scale = getScale(resolution);
    }
    if (scale < TEXT_CUTOFF_SCALE) {
      return undefined;
    }
    return [
      getScaledElementStyle(this.polygonStyle, this.strokeStyle, scale),
      getScaledElementStyle(this.style, undefined, scale),
    ];
  }
}
